function solution = BranchAndBound(problem)

%Solves an optimization problem using a branch and bound algorithm
%
% USAGE: solution = BranchAndBound(problem)
%
% PARAMETERS:
%   problem ~ structure that must contain the following fields
%       costFunction ~ the function to optimize.
%           ----------------------------------------------------------
%           WARNING: The cost function is expected of the form
%                 [flag, fValue] = costFunction(x)
%           So first return value is a flag indicating wether we could
%           compute the costFunction at point x or not, and the second
%           return value is the costFunction at x.
%           ----------------------------------------------------------
%       constantLipschitz ~ the Lipschitz constant of the function
%       startingPoint ~ the center of the starting cube
%       startingSize ~ the size (one per dimension) of the starting cube
%     It may also containt the following fields
%       upperBound ~ an initial upper bound (default: inf)
%       tolerance ~ required tolerance (default: startingSize/10)
%       maxBranches ~ maximum number of branches (default: 1000)
%
% RETURN:
%   solution ~ structure with two fields
%       potentialList ~ list of potential regions
%       discardedList ~ list of discarded regions
%       

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    % Check problem structure, mandatory and default optional fields.
    problem = checkOptions(problem);
    
    % Cost function and lipschitz constant
    f = problem.costFunction;
    L = problem.constantLipschitz;
    
    % Define the entry point to the problem
    X0 = problem.startingPoint;
    
    % Define the algorithm variables
    % Lists
    potentialList = X0;
    discardedList = [];
    % Level
    lv = 1;
    % Current size
    s = problem.startingSize;
    if numel(s) == 1,
        s = s*ones(size(X0));
    end
    % Number of branches
    nb = 0;
    
    % Initial value for optimal: the minimum across X0
    f0 = f(X0);
    % Get minimum value
    [u,index] = min(f0);
    % Get coordinate
    optimalPoint = X0(:,index);
    
    % Main loop, bound and branch
    while max(s) > problem.tolerance && nb < problem.maxBranches,
        % Compute bounds
        [~,lB] = lowerBound(f,potentialList,s,L);
        [uFlag,uB] = upperBound(f,potentialList,s,L);
        
        % Remove those who are far above the bound
        [auxU] = min(uB(~uFlag));
        if auxU < u,
            optimalPoint = potentialList(:,uB==auxU);
            u = auxU;
        end
%         discardedList = cat(1,discardedList,[potentialList(:,lB > u); lv*ones(sum(lB>u),1)]);
        potentialList(:,lB > u) = [];
        
        % Increase nb and branch
        nb = nb + size(potentialList,2);
        
        % Process all the elements in list
        potentialList = branch(potentialList,s);

        % Update s and lv
        s = s/2;
        lv = lv + 1;
    end

    % Give solution
    if ~isempty(optimalPoint),
        potentialList = cat(2,optimalPoint,potentialList);
    end
    solution.potentialList = potentialList;
    solution.discardedList = discardedList;

end

% Branch
function olist = branch(ilist,s)
    
    % Define
    olist = [];
    
    % Variables
    dim = size(ilist,1);
    s = s/2;
    
    % MyBasis
    B = (2*dec2bin(0:2^dim-1,dim)-97)';

    % Constrained problem
    for bb = 1:size(B,2),
        v = s.*B(:,bb);
        % Add first point
        x = ilist + repmat(v,1,size(ilist,2));
        % Cat
        olist = cat(2,olist,x);
    end        

end

% Upper bound function
function [flag,u] = upperBound(f,x,s,L)
    [flag,u] = f(x);
%     u = myV + L*norm(s)*sqrt(2);
end

% Lower bound function
function [flag,l] = lowerBound(f,x,s,L)
    [flag,myV] = f(x);
    l = myV - L*norm(s)*sqrt(2);
end

% Check problem structure, mandatory and default optional fields.
function p = checkOptions(p)

    % Check the mandatory fields of problem
    mandatoryFields = {...
        'costFunction','constantLipschitz',...
        'startingPoint','startingSize'};
    for f = 1:numel(mandatoryFields),
        if ~isfield(p,mandatoryFields{f})
            error(['Field ' mandatoryFields{f} ' is mandatory.']);
        end
    end
    
    % Default options
    % Field names
    defaultValues = {...
        {'upperBound',      inf    },...
        {'tolerance',       min(p.startingSize)/10  },...
        {'maxBranches',         1000          },...
        };
    
    % Loop on it
    for no = 1:numel(defaultValues),
        if ~isfield(p,defaultValues{no}{1})
            p.(defaultValues{no}{1}) = defaultValues{no}{2};
        end
    end
    
end