function [solverOutput, methodOptions] = CancliniTDE(XCPC, microphonePositions, samplingPeriod, x0, methodOptions)

%Finds the best sound source position given the independently estimated
%time delays.
%
% USAGE: [solverOutput, methodOptions] = CancliniTDE(XCPC, microphonePositions, samplingPeriod, x0, methodOptions)
%
% PARAMETERS: 
%   XCPC ~ the interpolation coefficients of the cross-correlation function
%   microphonePositions ~ positions of the microphones
%   samplingPeriod ~ sampling period of the signal
%   x0 ~ initialization positions for the local optimizer
%   methodOptions ~ method options
%
% RESULT:
%   solverOutput ~ the solution of the solver
%   methodOptions ~ the method options eventually modified
%
% DESCRIPTION:
%   The time delays are extracted independently using a maximum
%   cross-correlation criterion. Later, the sound source position fitting
%   the best the extracted time delays is computed by local minimization
%   initilized on a grid.
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    %%% Input check
    if nargin < 5
        error('Usage [solverOutput, methodOptions] = CancliniTDE(XCPC, microphonePositions, samplingPeriod, x0, methodOptions)');
    end
    if size(XCPC,1) ~= size(microphonePositions,1)
        error('There should be as many signals as microphones.');
    end
    if size(microphonePositions,2) ~= 3 %&& size(microphones,2) ~= 2
        error('The system works only in 3D.');
    end
    if samplingPeriod <= 0
        error('The sampling period should be positive.');
    end
    
    %%% General vars
    NMics = size(microphonePositions,1);
    
    %%% Aux variable
    TDEmeasures = nan(1,(NMics-1)*(NMics)/2);
    
    %%% Look for the maxima in each TDE
    for mic1= 1:NMics,
        for mic2 = mic1+1:NMics,
            % TDE Index
            TDEIndex = TDEis(mic1,mic2,NMics);
            % Declare the objective function
            objFunction = @(x) CrossCorrelationInterpolation(XCPC{mic1,mic2},x,samplingPeriod);
            % Evaluate at X0
            f = objFunction(x0{TDEIndex});
            % Compute the local maximas
            ind = false(size(f));
            ind(2:end-1) = f(2:end-1)>f(1:end-2) & f(2:end-1)>f(3:end);
            if sum(ind)
                % If there are local maxima
                % My F and my X
                myF = f(ind);
                myX = x0{TDEIndex}(ind);
                % Maximas
                [~,ind2] = sort(myF,'descend');
                myX = myX(ind2);
                % Save
                TDEmeasures(TDEIndex) = myX(1);
            else
                % Otherwise, take the highest extreme
                [~,ind] = max(f);
                TDEmeasures(TDEIndex) = x0{TDEIndex}(ind);
            end
        end
    end
    
    % Reset microphones and positions
    offset = mean(methodOptions.initPos,1);
    centeredInitPos = (methodOptions.initPos-repmat(offset,size(methodOptions.initPos,1),1))';
    offset = mean(microphonePositions,1);
    centeredMicrophones = microphonePositions - repmat(offset,NMics,1);
    lambda = 10;
    % Declare objective function
    objFunction = @(x) CriterionCanclini(x,TDEmeasures,centeredMicrophones,lambda,methodOptions.rho);
    % Declare dummy constraint function
    consFunction = @(x,z) DummyConstraint(x,z);

    % Run the parallel local optimization
    [solverOutput, methodOptions] = IPSolverParallelOA(centeredInitPos,objFunction,consFunction,methodOptions);
    
    % Compute the direct projection into TDE
    TDEAux = TDEGeometricDirect(solverOutput.x',centeredMicrophones);
    solverOutput.x = TDEAux(:,1:3)';
    
    
end

function [FJ, J, GJ, HJ] = CriterionCanclini(Pos,TDEs,Mics,lambda,rho)
    %%% Input check
    if nargin < 3
        error('Usage: [FJ, J, GJ, HJ] = CriterionCanclini(Pos,TDEs,Mics)');       
    end
    if nargin < 5,
        lambda = 0;
        rho = 1;
    end
    Dimension = 3;
    % Number of microphones
    NMics = size(Mics,1);
    % TDEs dimension and number
    NTDEs = numel(TDEs);
    
    % ==================================
    % Scale TDE with the sound speed!!!!
    % ==================================
    TDEs = 343.2*TDEs;
    % ==================================
    
    % Neval
    NEval = size(Pos,2);
    % Exit flag
    FJ = zeros(1,NEval);
    
    % Declare partial
    J = zeros(1,NEval,NTDEs);
    GJ = zeros(Dimension,NEval,NTDEs);
    HJ = zeros(Dimension,Dimension,NEval,NTDEs);
    
    % Loop on the microphones' index
    for mic1 = 1:NMics,
        for mic2 = mic1+1:NMics,
            % Get TDE Index
            TDEIndex = TDEis(mic1,mic2,NMics);
            % Auxiliar variables
            V = TDEs(TDEIndex)^2;
            A = V + norm(Mics(mic1,:),2)^2 - norm(Mics(mic2,:),2)^2;
            % Precompute
            DM = Mics(mic1,:) - Mics(mic2,:);
            % Compute J
%             J(1,:,TDEIndex) = A^2 - 4*V*norm(Mics(mic1,:))^2;
            % Loop on the dimension
            for d1 = 1:Dimension,
                % Update J
                J(1,:,TDEIndex) = J(1,:,TDEIndex) + ...
                    4*(A*DM(d1) + 2*V*Mics(mic1,d1))*Pos(d1,:);
                % Set GJ
                GJ(d1,:,TDEIndex) = 4*(A*DM(d1) + 2*V*Mics(mic1,d1));
                % Loop on the dimension (again)
                for d2 = 1:Dimension,
                    % Depending on d1=d2 update J and HJ accordingly
                    if d2 == d1,
                        % Update J with quadratic terms
                        J(1,:,TDEIndex) = J(1,:,TDEIndex) + ...
                            4*(DM(d1)^2-V)*Pos(d1,:).^2;
                        % Update GJ with linear terms
                        GJ(d1,:,TDEIndex) = GJ(d1,:,TDEIndex) + ...
                            8*(DM(d1)^2-V)*Pos(d1,:);
                        % Set the diagonal of HJ
                        HJ(d1,d1,:,TDEIndex) = 8*(DM(d1)^2-V);
                    else
                        % Update J with the cross-product terms
                        J(1,:,TDEIndex) = J(1,:,TDEIndex) + ...
                            (4*DM(d1)*DM(d2)*Pos(d1,:).*Pos(d2,:));
                        % Update GJ with cross terms
                        GJ(d1,:,TDEIndex) = GJ(d1,:,TDEIndex) + ...
                            8*DM(d1)*DM(d2)*Pos(d2,:);
                        % Set the non-diagonal of HJ
                        HJ(d1,d2,:,TDEIndex) = 8*DM(d1)*DM(d2);
                    end
                end    
            end
            % Loop on the dimension to set the correct hessian
            for d1 = 1:Dimension,
                for d2 = 1:Dimension,
                    % Update HJ
                    HJ(d1,d2,:,:) = squeeze(HJ(d1,d2,:,:)) .* squeeze(J) + squeeze(GJ(d1,:,:).*GJ(d2,:,:));
                end
            end
            % Loop on the dimension to set the correct gradient
            for d1 = 1:Dimension,
                % Updage GJ
                GJ(d1,:,:) = GJ(d1,:,:) .* J;
            end
        end
    end
    
    % Sum along the TDEs and squeeze
    J = squeeze(sum(J.^2,3));
    GJ = 2*squeeze(sum(GJ,3));
    HJ = 2*squeeze(sum(HJ,4));
    
    % Constraint
    C = (sum(Pos.^2,1)-rho^2);
    J = J + lambda*C.^2;

    GJ = GJ + lambda*4*repmat(C,Dimension,1).*Pos;
    for d1 = 1:Dimension,
        for d2 = 1:Dimension,
            if d1 == d2,
                HJ(d1,d2,:) = squeeze(HJ(d1,d2,:)) + lambda*16*Pos(d1,:).^2';
            else
                HJ(d1,d2,:) = squeeze(HJ(d1,d2,:)) + (lambda*8*Pos(d1,:).*Pos(d2,:))';
            end
        end
    end

end
