function [XC, XCd, XCdd, flag] = CrossCorrelationInterpolation(XCPC,Delay,SamplingPeriod,ZeroIndex)

%Interpolate the cross-correlation function (and its derivatives).
%
% USAGE: [XC, XCd, XCdd, flag] = CrossCorrelationInterpolation(XCPC,Delay,SamplingPeriod[,ZeroIndex])
%
% PARAMETERS:
%   XCPC ~ cross-correlation signals of the polynomial coefficients.
%   Delay ~ delay is the point(s) in which we want to estimate the signals'
%     cross-correlation function.
%   SamplingPeriod ~ the signals' sampling period.
%   ZeroIndex ~ the index on the XCPC representing no delay (the default value is ceil(L/2), where L is the
%     size of the coefficients cross-correlation function.)
% 
% RETURN VALUE:
%   XC, XCd, XCdd ~ The value of the cross-correlation function, its first and
%     its second derivative.
%   flag is true for those value of Delay in which the cross correlation
%   cannot be evaluated.
% 
% DESCRIPTION:
%     This function estimates the value of the cross-correlation function
%     at Delay from from the polynomial coefficients' cross-correlation 
%     signals XCPC.
%
% see also Signals2XCPC, 
%    

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhone-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    % Input check
    if nargin < 3
        error('Usage: [XC, XCd, XCdd, flag] = CrossCorrelationInterpolation(PCCC,Delay,SamplingPeriod[,ZeroIndex])');
    end
    if nargin < 4
        ZeroIndex = ceil(size(XCPC,2)/2);
    end

    % Output value
    XC = zeros(size(Delay));
    XCd = zeros(size(Delay));
    XCdd = zeros(size(Delay));
    if nargout > 3,
        flag = zeros(size(Delay));
    end
    
    % Index in the cross-correlation
    ccIndex = floor(Delay/SamplingPeriod);
    % Remaining of the division (0 <= tau < SamplingPeriod)
    tau = Delay - ccIndex*SamplingPeriod;
    
    % Interpolation polynomial degrees
    D = size(XCPC,1)-1;
    
    if ismatrix(XCPC),
        % Which indices are inside?
        ccIndex = ccIndex + ZeroIndex;
        inside = ccIndex > 0 & ccIndex <= size(XCPC,2);
        % Set the correspondent flag
        flag(~inside) = 1;
        % No points to evaluate
        if sum(inside) == 0
            return;
        end
        % Reduce the taus
        tau = tau(inside);
        % Allocate space
        cc = zeros(size(tau));
        ccd = zeros(size(tau));
        ccdd = zeros(size(tau));
        for d = 0:D,
            % Cross-correlation's value
            cc = cc + XCPC(d+1,ccIndex(inside)).*tau.^d;
            % Cross-correlation's derivatives
            if d > 0
                ccd = ccd + d*XCPC(d+1,ccIndex(inside)).*tau.^(d-1);
            end
            % Cross-correlation's second derivative
            if d > 1
                ccdd = ccdd + d*(d-1)*XCPC(d+1,ccIndex(inside)).*tau.^(d-2);
            end
        end
        % Store good values
        XC(inside) = cc;
        XCd(inside) = ccd;
        XCdd(inside) = ccdd;
    else
        error('Dimension of XCPC should be 2');
    end
return

