function [TDEs, inside] = DiscardOutBounds(TDEs,microphonesPosition)

%Discard those TDE vectors that lay out of the theoretic bounds.
%
% USAGE: [TDEs, inside] = DiscardOutBounds(TDEs,microphonesPosition)
%
% PARAMETERS:
%   TDEs ~ Time Delay Values for all the time delays corresponding to the
%     first microphone.
%   microphonesPosition ~ Positions of the microphones.
%
% RETURN:
%   TDEs ~ The TDE vectors that are geometrically possible.
%   inside ~ If asked, a boolean vector indicating which of the input TDEs
%     are inside the bounds.
%
% DESCRIPTION:
%   The function checks that the TDE values lay inside the bounds imposed
%   by the physics of the sound propagation. The maximum value of the TDE
%   corresponding to the (i,j)-th microphone pair is the euclidean distance
%   between the i-th and the j-th microphone divivded by the sound speed.
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    % TDEs dimension and quantity
    [Dimension] = size(TDEs,1);
    % Number of microphones
    NMics = size(microphonesPosition,1);

    % Check
    if Dimension ~= NMics-1,
        error('Dimension should be NMics-1.');
    end
    
    % inside
    indices = true(1,size(TDEs,2));

    % Maximum TDEs
    maxTDE = TDEmax(microphonesPosition);

    % Loop with mic1=1
    for mic2 = 2:NMics,
        % Get the index
        TDEindex = TDEis(1,mic2,NMics);
        % Indices
        indices = indices & inBoundTest(TDEs(TDEindex,:),maxTDE(TDEindex));
    end

    % Loop
    for mic1 = 2:NMics,
        for mic2 = mic1+1:NMics,
            % Get the index
            TDEindex1 = TDEis(1,mic1,NMics);
            TDEindex2 = TDEis(1,mic2,NMics);
            % TDEs to bount
            auxTDE = TDEs(TDEindex2,:)-TDEs(TDEindex1,:);
            % Indices
            indices = indices & inBoundTest(auxTDE,maxTDE(TDEindex));
        end
    end
    
    % Get them
    TDEs = TDEs(:,indices);
    % If inside asked
    if nargout > 1
        inside = indices;
    end

end

function indices = inBoundTest(values,bound)
    indices = (values >= -bound) & (values <= bound);
end