function L = EstimateLipschitzConstant(microphonesPosition,samplingPeriod,functionHandle)

%Estimates the Lipschitz constant of functionHandle.
%
% USAGE: L = EstimateLipschitzConstant(microphonesPosition,samplingPeriod,functionHandle)
%
% PARAMETERS:
%   microphonesPosition ~ positions of the microphones
%   samplingPeriod ~ sampling period
%   functionHandle ~ function handle
%
% RETURN VALUE:
%   L ~ estimate of the Lipschitz constant
% 
% DESCRIPTION:
%   L is estimated by generating N=1000 points in the feasibility domain
%   and computing max(|f(x_j)-f(x_i)|/||x_j-x_i||) over i and j.
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhone-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    %%% Input check
    if nargin < 2
        error('Usage: L = EstimateLipschitzConstant(microphonesPosition,samplingPeriod,functionHandle).');
    end
    
    %%% Random estimation
    Ntdes = 1000;
    % Maximum TDE values from the microphones
    maxTDE = TDEmax(microphonesPosition);
    
    % Generate random tdes
    RTDEs = rand(size(microphonesPosition,1)-1,Ntdes)-0.5;
    for n = 1:size(microphonesPosition,1)-1,
        RTDEs(n,:) = 2*maxTDE(n)*RTDEs(n,:)/samplingPeriod;
    end
    
    % Evaluate f
    J = functionHandle(RTDEs);

    % Compute the lipschitz constant for each pair and take the max
    L = 0;
    for n = 1:Ntdes,
        for m = n+1:Ntdes,
            auxL = abs(J(m)-J(n))/norm(RTDEs(:,m)-RTDEs(:,n));
            if auxL > L,
                L = auxL;
            end
        end
    end
    
end
    