function [solverOutput, op] = IPSolverParallelOA (x, objective, constraint, op)

%Interior-Point algorithm for several initializations at once
%
% USAGE: [solverOutput, op] = IPSolverParallel (x, objective, constraint[, op])
%
% PARAMETERS:
%  x ~ is the (d,n) matrix of the n initializations of dimesion d.
%
%  objective(x) ~ it is the handler of the objective function. This handler
%       should return in this order:
%       eF ~ n-vector 0 if objective is evaluable at x, >0 otherwise
%       f ~ n-vector with objective at x
%       g ~ (d,n) matrix with the gradients of objective at x
%       H ~ (d,d,n) matrix with the Hessians at x
%
%  constraint(x,z) ~ it is the handler for the constraints. This handler should
%       return, in this order (z is a n-vector)
%       c ~ n-vector with the constraint values at x
%       J ~ (d,n) matrix with the value of the Jacobian of the
%       constraint at x
%       W ~ (d,d,n) matrix with the value of the sum of Hessian of the
%       constraint at x, that is:
%           W(:,:,i) = z(i)Hc(i)
%
%  op ~ structure of options: option name [default value]
%         descentdir      [newton] (only choice)
%         tolerance       [1e-6]   (if the point moves less, does not move)
%         maxiter         [10]     (maximum number of iterations)
%         verbose         [false]
%         condtest        [false]  (invertibility test)
%         eps             [1e-8]   (machine zero)
%       parameters of the IP algorithm & backtrack search see refences
%         sigmamax        [0.5]
%         etamax          [0.25]
%         mumin           [1e-9]
%         alphamax        [0.995]
%         alphamin        [1e-4]
%         beta            [0.75]
%         tau             [0.01]
%
% RETURN VALUE:
%  solverOutput ~ structure with the following fields
%    x ~ the matrix of output values
%    f ~ the vector of objective(x)
%    c ~ the vector of constraint(x)
%    flag ~ exitFlag (0,converged), (1,max iterations reached), 
%           (2,non-invertibility of the perturbed KKT system), 
%           (<0, not able to compute objective function)
%    nIter ~ number of iterations till exit
%    [IF op.output == 2]
%    xIT ~ x at each iteration
%    fIT ~ f at each iteration
%    cIT ~ c at each iteration
%    r0IT ~ residual at each iteration
%    psiIT ~ merit function at each iteration
%    gradpsiIT ~ gradient of the merit function at each iteration
%
%  op ~ method options at the end
% 
% DESCRIPTION:
%     This is a parallelized version of the algorithm described and
%     implemented in [Carbonetto 2008]. The code for the non-parallelized 
%     version is also available online.
%
% REFERENCES:
%     P. Carbonetto, MATLAB primal-dual interior-point solver for convex 
%     programs with constraints, 2008.
%     http://www.cs.ubc.ca/ pcarbo/convexprog.html.
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    if nargin < 3,
        error('Usage: [solverOutput, op] = IPSolverParallel (x, objective, constraint[, op])');
    end
    
    if nargin < 4
        op = struct();
    end
    op = check_options(op);

    % INITIALIZATION.  
    % Get the number of primal variables (n), the number of constraints
    % (m), the number of initializations (ni)
    % and the total number of primal-dual optimization variables (nv).
    % Initialize the Lagrange multipliers. Initialize the second-order
    % information.
    [n, ni] = size(x);
    
    %%% Output
    % Which point has already converged?
    finalX = zeros(size(x));
    % Exit flag, Number of interations and active indices
    exitFlag = ones(ni,1);
    nIter = zeros(ni,1);
    activeIndices = 1:ni;
    % Verbose output
    if op.output == 2,
        xIT = [];
        fIT = [];
        cIT = [];
        r0IT = [];
        psiIT = [];
        gradpsiIT = [];
    end
    
    %%%% FORCING to 1 constraint
    m  = 1;
    z  = ones(m,ni);
    nv = n + m;

    if op.verbose
        fprintf('  i f(x)       lg(mu) sigma   ||rx||  ||rc||  alpha   #ls\n');
    end

    % Repeat while the convergence criterion has not been satisfied, and
    % we haven't reached the maximum number of iterations.
    saved = false(1,ni);
    
    for iter = 1:op.maxiter
        % COMPUTE OBJECTIVE, GRADIENT, CONSTRAINTS, ETC.  
        % Compute the response of the objective function, the gradient of the
        % objective, the response of the inequality constraints, the Jacobian of
        % the inequality constraints, the Hessian of the Lagrangian (minus the
        % Hessian of the objective) and, optionally, the Hessian of the
        % objective.
        if strcmp(op.descentdir,'newton')
            [eF, f, g, B] = objective(x);
            % Those that we cannot evaluate, stop
            nTSLocRef = eF > 0;
            nTSGlobRef = activeIndices(nTSLocRef);
            exitFlag(nTSGlobRef) = -eF(nTSLocRef);
            % Mark as saved and save
            saved(nTSGlobRef) = true;
            finalX(:,nTSGlobRef) = x(:,nTSLocRef);
        else
            error('Do not do non-newton.');
        end
        
        % Keep the non-saved! Be careful here!!!!
        % Keep in mind that x (and hence eF) do not correspond in indices
        % to saved. So the following happens: we keep the non saved, but we
        % recover their position in the vector x (not in the vector saved).
        toKeep = ~saved(activeIndices);
        x = x(:,toKeep); eF = eF(toKeep); f = f(toKeep); g = g(:,toKeep);
        B = B(:,:,toKeep); activeIndices = activeIndices(toKeep); z = z(toKeep);
        
        % Compute constraint(s) and derivatives
        [c, J, W] = constraint(x,z);

        % Compute the responses of the unperturbed Karush-Kuhn-Tucker
        % optimality conditions.
        rx = g + J.*repmat(z,n,1);  % Dual residual.
        rc = c.*z;      % Complementarity.
        r0 = [rx; rc]; 
        
        % Set some parameters that affect convergence of the primal-dual
        % interior-point method.
        auxr0 = sqrt( sum( r0.^2, 1))/nv;
        eta        = min(op.etamax,auxr0);
        sigma      = min(op.sigmamax,sqrt(auxr0));
        dualitygap = -c.*z;
        mu         = max(op.mumin,sigma.*dualitygap/m);

        % CONVERGENCE CHECK.
        % (If the norm of the responses is less than the specified
        % tolerance or we could not compute the oF at the point)
        % and we have not saved the point before, save it!
        nTSLocRef = (auxr0 < op.tolerance);
        nTSGlobRef = activeIndices(nTSLocRef);
        exitFlag(nTSGlobRef) = 0;
        % Mark as saved and save
        saved(nTSGlobRef) = true;
        finalX(:,nTSGlobRef) = x(:,nTSLocRef);
        % If everyone is done, return
        if sum(~saved) == 0
            break;
        end
       
        % Increase the number of iterations
        nIter(~saved) = nIter(~saved) + 1;

        % SOLUTION TO PERTURBED KKT SYSTEM.
        % Compute the search direction of x and z.
        S  = z./(c-op.eps);
        gb = g - repmat(mu,n,1).*J.*(1./(repmat(c,n,1)-op.eps));
        px = zeros(n,numel(activeIndices));
        % Loop on the activeIndices and check they did not converge yet
        for jj = 1:numel(activeIndices),
            iii = activeIndices(jj);
%         for iii = 1:ni,
            if ~saved(iii)
%                 if op.condtest && det(B(:,:,iii) + W(:,:,iii) - J(:,iii)*S(iii)*J(:,iii)') < op.eps
                if abs(det(B(:,:,jj) + W(:,:,jj) - J(:,jj)*S(jj)*J(:,jj)')) < eps
                    finalX(:,iii) = x(:,jj);
                    saved(iii) = true;
                    % Mark the non-invertibility
                    exitFlag(iii) = 2;
                else     
                    px(:,jj) = (B(:,:,jj) + W(:,:,jj) - J(:,jj)*S(jj)*J(:,jj)') \ (-gb(:,jj));
                end
            end
        end
        % If everyone is done, return
        if sum(~saved) == 0
            break;
        end        
        pz = -(z + mu./(c-op.eps) +  S.*sum(J.*px,1));
        
        
        
        % REMAP BEFORE BACKTRACKING
        toKeep = ~saved(activeIndices);
        x = x(:,toKeep); f = f(toKeep); g = g(:,toKeep);
        B = B(:,:,toKeep); activeIndices = activeIndices(toKeep); z = z(toKeep);
        c = c(toKeep); J = J(:,toKeep);
        pz = pz(toKeep); px = px(:,toKeep);
        mu = mu(toKeep); eta = eta(toKeep);

        % BACKTRACKING LINE SEARCH.
        % To ensure global convergence, execute backtracking line search to
        % determine the step length. First, we have to find the largest step
        % size which ensures that z remains feasible. Next, we perform
        % backtracking line search.
        
        % Number of points being backtraced
        bni = numel(activeIndices);
        alpha = op.alphamax*ones(1,bni);
        is    = find(z + pz < 0);
        if ~isempty(is)
            alpha(is) = op.alphamax * min(1, z(is) ./ -pz(is) );
        end

        % Compute the response of the merit function and the directional
        % gradient at the current point and search direction.
        psi  = merit(x,z,f,c,mu,op.eps);
        dpsi = gradmerit(x,z,px,pz,g,c,J,mu,op.eps);
        ls   = zeros(1,bni);
        backtrack = true(1,bni);
        savedAmongActive = saved(activeIndices);
        while true
            % Backtrack those points which are not saved before
            backtrack = ~savedAmongActive & backtrack;
            % This condition is necessary in case all the points in the
            % last inner iteration are either stopped begin backtracked or
            % saved!
            if sum(backtrack) == 0
                break;
            end
            % Compute the candidate point, the constraints, and the response of
            % the objective function and merit function at the candidate point.
            
            ls = ls + 1;
            % Initialize
            xnew = zeros(n,bni);
            znew = zeros(1,bni);
            psinew = zeros(1,bni);
            % Compute
            xnew(:,backtrack) = x(:,backtrack) + repmat(alpha(backtrack),n,1) .* px(:,backtrack);
            znew(backtrack) = z(backtrack) + alpha(backtrack) .* pz(backtrack);
            [eF,faux] = objective(xnew);
            f(backtrack) = faux(backtrack);
            c(backtrack) = constraint(xnew(:,backtrack),[]);
            psinew(backtrack) = merit(xnew(:,backtrack),znew(backtrack),f(backtrack),c(backtrack),mu(backtrack),eps);
            
            % If there is an evaluation error, we should stop the
            % backtracking and save the point
            newToSave = (eF > 0) & ~savedAmongActive & backtrack;
            exitFlag(newToSave) = -eF(newToSave);
            % Mark as saved and save
            saved(activeIndices(newToSave)) = true;
            finalX(:,activeIndices(newToSave)) = x(:,newToSave);
            backtrack = backtrack & ~newToSave;

            % Stop backtracking search if we've found a candidate point that
            % sufficiently decreases the merit function and satisfies all the
            % constraints (and if we were backtracking it, of course)
            stopBacktracking = (~(c > 0)) & psinew < (psi + op.tau*(eta.*alpha).*dpsi);
            stopBacktracking = stopBacktracking & backtrack;
            % For those which have to stop, save the point
            x(:,stopBacktracking)     = xnew(:,stopBacktracking);
            z(stopBacktracking)     = znew(stopBacktracking);
            
            % Candidate points which do not meet our criteria, will have a
            % smaller alpha next backtracking search
            alpha(~stopBacktracking) = alpha(~stopBacktracking) * op.beta;
            % Mark the candidates which were backtracked and that have
            % a too small alpha
            newToSave = alpha<op.alphamin & backtrack;
            % Save in the exit flag
            exitFlag(activeIndices(newToSave)) = 3;
            
            % If there are points to be backtracked, still, save those who
            % had a too small alpha
            finalX(:,activeIndices(newToSave)) = xnew(:,newToSave);
            saved(activeIndices(newToSave)) = true;  
            % If everyone is done, return
            if sum(~saved) == 0
                break;
            end
            
            % Update backtrack variable, with those who do not have to be
            % backtracked any more, and check if there are still some
            % poitns to be backtracked
            backtrack = backtrack & ~stopBacktracking & ~newToSave;
        end
        
        % If needed for the output
        if op.output == 2,
            xAux = x;
            xAux(:,saved) = finalX(:,saved);
            xIT = cat(3,xIT,xAux);
            fIT = cat(3,fIT,f);
            cIT = cat(3,cIT,c);
            r0IT = cat(3,r0IT,r0);
            psiIT = cat(3,psiIT,psi);
            gradpsiIT = cat(3,gradpsiIT,dpsi);
        end
        
        % Check if done
        if sum(~saved) == 0
            break;
        end
        
        % REMAP
        toKeep = ~saved(activeIndices);
        x = x(:,toKeep); activeIndices = activeIndices(toKeep); z = z(toKeep);
    end
    % Save the final results of those initializations which did not
    % converge
    if sum(~saved) ~= 0
        finalX(:,~saved) = x;
    end
    % Compute OJ and C
    [~,f] = objective(finalX);
    c = constraint(finalX,[]);
    % Build output
    solverOutput.x = finalX;
    solverOutput.f = f;
    solverOutput.c = c;
    solverOutput.flag = exitFlag;
    solverOutput.nIter = nIter;
    % If need more...
    if op.output == 2
        solverOutput.xIT = xIT;
        solverOutput.fIT = fIT;
        solverOutput.cIT = cIT;
        solverOutput.r0IT = r0IT;
        solverOutput.psiIT = psiIT;
        solverOutput.gradpsiIT = gradpsiIT;
    end
end
  
% ------------------------------------------------------------------
% Compute the response of the merit function at (x,z).
function psi = merit (~, z, f, c, mu, eps)
    psi = f - c.*z - mu.*log(c.^2.*z+eps);
end
  
% ------------------------------------------------------------------
% Compute the directional derivative of the merit function at (x,z).
function dpsi = gradmerit (~, z, px, pz, g, c, J, mu, eps)
    dpsi = sum(px.*(g - J.*repmat(z,size(J,1),1) - 2*J.*repmat(mu./(c-eps),size(J,1),1)),1) -...
        pz.*(c + mu./(z+eps));
end

function options = check_options(options)
    % Field names
    defaultValues = {...
        {'descentdir',      'newton'    },...
        {'tolerance',       1e-6        },...
        {'maxiter',         10          },...
        {'verbose',         false       },...
        {'condtest',        false       },...
        {'eps',             1e-8        },...
        {'sigmamax',        0.5         },...
        {'etamax',          0.25        },...
        {'mumin',           1e-9        },...
        {'alphamax',        0.995       },...
        {'alphamin',        1e-4        },...
        {'beta',            0.75        },...
        {'tau',             0.01        }...
        {'output',          0           }...
        };
    
    % Loop on it
    for no = 1:numel(defaultValues),
        if ~isfield(options,defaultValues{no}{1})
            options.(defaultValues{no}{1}) = defaultValues{no}{2};
        end
    end
end